
import { localize } from '../../../../nls.js';
import { RawContextKey } from '../../../../platform/contextkey/common/contextkey.js';

var ChatContextKeys;
(function (ChatContextKeys) {
    ChatContextKeys.responseVote = ( new RawContextKey(
        'chatSessionResponseVote',
        '',
        { type: 'string', description: ( localize(
            4607,
            "When the response has been voted up, is set to 'up'. When voted down, is set to 'down'. Otherwise an empty string."
        )) }
    ));
    ChatContextKeys.responseDetectedAgentCommand = ( new RawContextKey(
        'chatSessionResponseDetectedAgentOrCommand',
        false,
        { type: 'boolean', description: ( localize(4608, "When the agent or command was automatically detected")) }
    ));
    ChatContextKeys.responseSupportsIssueReporting = ( new RawContextKey(
        'chatResponseSupportsIssueReporting',
        false,
        { type: 'boolean', description: ( localize(4609, "True when the current chat response supports issue reporting.")) }
    ));
    ChatContextKeys.responseIsFiltered = ( new RawContextKey(
        'chatSessionResponseFiltered',
        false,
        { type: 'boolean', description: ( localize(4610, "True when the chat response was filtered out by the server.")) }
    ));
    ChatContextKeys.responseHasError = ( new RawContextKey(
        'chatSessionResponseError',
        false,
        { type: 'boolean', description: ( localize(4611, "True when the chat response resulted in an error.")) }
    ));
    ChatContextKeys.requestInProgress = ( new RawContextKey(
        'chatSessionRequestInProgress',
        false,
        { type: 'boolean', description: ( localize(4612, "True when the current request is still in progress.")) }
    ));
    ChatContextKeys.isResponse = ( new RawContextKey('chatResponse', false, { type: 'boolean', description: ( localize(4613, "The chat item is a response.")) }));
    ChatContextKeys.isRequest = ( new RawContextKey('chatRequest', false, { type: 'boolean', description: ( localize(4614, "The chat item is a request")) }));
    ChatContextKeys.itemId = ( new RawContextKey('chatItemId', '', { type: 'string', description: ( localize(4615, "The id of the chat item.")) }));
    ChatContextKeys.lastItemId = ( new RawContextKey('chatLastItemId', [], { type: 'string', description: ( localize(4616, "The id of the last chat item.")) }));
    ChatContextKeys.editApplied = ( new RawContextKey('chatEditApplied', false, { type: 'boolean', description: ( localize(4617, "True when the chat text edits have been applied.")) }));
    ChatContextKeys.inputHasText = ( new RawContextKey('chatInputHasText', false, { type: 'boolean', description: ( localize(4618, "True when the chat input has text.")) }));
    ChatContextKeys.inputHasFocus = ( new RawContextKey('chatInputHasFocus', false, { type: 'boolean', description: ( localize(4619, "True when the chat input has focus.")) }));
    ChatContextKeys.inChatInput = ( new RawContextKey('inChatInput', false, { type: 'boolean', description: ( localize(4620, "True when focus is in the chat input, false otherwise.")) }));
    ChatContextKeys.inChatSession = ( new RawContextKey('inChat', false, { type: 'boolean', description: ( localize(4621, "True when focus is in the chat widget, false otherwise.")) }));
    ChatContextKeys.enabled = ( new RawContextKey('chatIsEnabled', false, { type: 'boolean', description: ( localize(
        4622,
        "True when chat is enabled because a default chat participant is activated with an implementation."
    )) }));
    ChatContextKeys.panelParticipantRegistered = ( new RawContextKey(
        'chatPanelParticipantRegistered',
        false,
        { type: 'boolean', description: ( localize(4623, "True when a default chat participant is registered for the panel.")) }
    ));
    ChatContextKeys.editingParticipantRegistered = ( new RawContextKey(
        'chatEditingParticipantRegistered',
        false,
        { type: 'boolean', description: ( localize(4624, "True when a default chat participant is registered for editing.")) }
    ));
    ChatContextKeys.chatEditingCanUndo = ( new RawContextKey(
        'chatEditingCanUndo',
        false,
        { type: 'boolean', description: ( localize(
            4625,
            "True when it is possible to undo an interaction in the editing panel."
        )) }
    ));
    ChatContextKeys.chatEditingCanRedo = ( new RawContextKey(
        'chatEditingCanRedo',
        false,
        { type: 'boolean', description: ( localize(
            4626,
            "True when it is possible to redo an interaction in the editing panel."
        )) }
    ));
    ChatContextKeys.extensionInvalid = ( new RawContextKey(
        'chatExtensionInvalid',
        false,
        { type: 'boolean', description: ( localize(
            4627,
            "True when the installed chat extension is invalid and needs to be updated."
        )) }
    ));
    ChatContextKeys.inputCursorAtTop = ( new RawContextKey('chatCursorAtTop', false));
    ChatContextKeys.inputHasAgent = ( new RawContextKey('chatInputHasAgent', false));
    ChatContextKeys.location = ( new RawContextKey('chatLocation', undefined));
    ChatContextKeys.inQuickChat = ( new RawContextKey('quickChatHasFocus', false, { type: 'boolean', description: ( localize(4628, "True when the quick chat UI has focus, false otherwise.")) }));
    ChatContextKeys.hasFileAttachments = ( new RawContextKey(
        'chatHasFileAttachments',
        false,
        { type: 'boolean', description: ( localize(4629, "True when the chat has file attachments.")) }
    ));
    ChatContextKeys.languageModelsAreUserSelectable = ( new RawContextKey(
        'chatModelsAreUserSelectable',
        false,
        { type: 'boolean', description: ( localize(4630, "True when the chat model can be selected manually by the user.")) }
    ));
    ChatContextKeys.Setup = {
        canSignUp: ( new RawContextKey('chatSetupCanSignUp', false, true)),
        signedOut: ( new RawContextKey('chatSetupSignedOut', false, true)),
        limited: ( new RawContextKey('chatSetupLimited', false, true)),
        triggered: ( new RawContextKey('chatSetupTriggered', false, true)),
        installed: ( new RawContextKey('chatSetupInstalled', false, true)),
    };
    ChatContextKeys.chatQuotaExceeded = ( new RawContextKey('chatQuotaExceeded', false, true));
    ChatContextKeys.completionsQuotaExceeded = ( new RawContextKey('completionsQuotaExceeded', false, true));
})(ChatContextKeys || (ChatContextKeys = {}));

export { ChatContextKeys };
